Module event

Module event 

Traits and helpers for the core Event type and event store implementations.

Re-exports

pub use publish::Publish;
pub use publish::PublishExt;
pub use replay::Replay;
pub use replay::ReplayExt;
pub use replay::ReplayOne;
pub use replay::ReplayOneExt;
pub use subscribe::Subscribe;
pub use subscribe::SubscribeExt;
pub use truncate::Truncate;

Modules

publish
Publish events to an event store.
replay
Replay existing events in an event store.
subscribe
Subscribe to new events from an event store.
truncate
Truncate (delete) old events from an event store.

Structs

Sequence
The relative order of an Event within a single event stream.

Traits

Event
Data representing a state change in the event sourced system.
EventGroup
Reference multiple event types in a single read stream.

Derive Macros

Event
Derive an Event implementation, with optional custom naming.
EventGroup
Derive an EventGroup implementation for an enum type.