EventGroup

Trait EventGroup 

pub trait EventGroup {
    // Required method
    fn names() -> impl Iterator<Item = &'static str>;
}
Reference multiple event types in a single read stream.

This is used to listen/replay messages for multiple aggregates from a single spot when projecting to a read model (rather than requiring multiple streams and readers for the same functionality).

Note that this trait is implemented automatically for all events, such that they are treated as a single-event group.

Required Methods

fn names() -> impl Iterator<Item = &'static str>

The names of all events within this Group.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors

Source

impl<E: Event> EventGroup for E