SerializeVersion

Trait SerializeVersion 

pub trait SerializeVersion: Serialize {
    // Required method
    fn version() -> usize;
}
Extend the serde Serialize trait with extra versioning info.

Required Methods

fn version() -> usize

Specify a version to be encoded alongside the serialized type.

Where this version info is stored depends on the event store backend.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors